@@ -1,7 +1,7 @@
-from mptypes import mpmathify, extraprec, eps, mpf
+from mptypes import mpmathify, extraprec, eps, mpf, MultiPrecisionArithmetic
 from calculus import diff
 from functions import sqrt, sign, ldexp
-from matrices import matrix, norm_p
+from matrices import matrix, norm as norm_
 from linalg import lu_solve
 from copy import copy
 
@@ -9,6 +9,48 @@ from copy import copy
 # 1D-SOLVERS #
 ##############
 
+class Newton:
+    """
+    1d-solver generating pairs of approximative root and error.
+
+    Needs starting points x0 close to the root.
+
+    Pro:
+
+    * converges fast
+    * sometimes more robust than secant with bad second starting point
+
+    Contra:
+
+    * converges slowly for multiple roots
+    * needs first derivative
+    * 2 function evaluations per iteration
+    """
+    maxsteps = 20
+
+    def __init__(self, f, x0, **kwargs):
+        if len(x0) == 1:
+            self.x0 = x0[0]
+        else:
+            raise ValueError('expected 1 starting point, got %i' * len(x0))
+        self.f = f
+        if not 'df' in kwargs:
+            def df(x):
+                return diff(f, x)
+        else:
+            df = kwargs['df']
+        self.df = df
+
+    def __iter__(self):
+        f = self.f
+        df = self.df
+        x0 = self.x0
+        while True:
+            x1 = x0 - f(x0) / df(x0)
+            error = abs(x1 - x0)
+            x0 = x1
+            yield (x1, error)
+
 class Secant:
     """
     1d-solver generating pairs of approximative root and error.
@@ -100,7 +142,7 @@ class MNewton:
             prevx = x
             fx = f(x)
             if fx == 0:
-		break
+                break
             dfx = df(x)
             d2fx = d2f(x)
             # x = x - F(x)/F'(x) with F(x) = f(x)/f'(x)
@@ -113,7 +155,7 @@ class Halley:
     1d-solver generating pairs of approximative root and error.
 
     Needs a starting point x0 close to the root.
-    Uses Halley's method with cubic convergance rate.
+    Uses Halley's method with cubic convergence rate.
 
     Pro:
 
@@ -212,7 +254,7 @@ class Muller:
         fx2 = f(x2)
         while True:
             # TODO: maybe refactoring with function for divided differences
-            # calculate divided diffferences
+            # calculate divided differences
             fx2x1 = (fx1 - fx2) / (x1 - x2)
             fx2x0 = (fx0 - fx2) / (x0 - x2)
             fx1x0 = (fx0 - fx1) / (x0 - x1)
@@ -304,7 +346,7 @@ class Illinois:
     """
     1d-solver generating pairs of approximative root and error.
 
-    Uses Illinois method or similair to find a root of f in [a, b].
+    Uses Illinois method or similar to find a root of f in [a, b].
     Might fail for multiple roots (needs sign change).
     Combines bisect with secant (improved regula falsi).
 
@@ -542,7 +584,7 @@ def jacobian(f, x):
 
 one = mpf(1)
 
-# TODO: support force_type
+# TODO: test with user-specified jacobian matrix, support force_type
 class MDNewton:
     """
     Find the root of a vector function numerically using Newton's method.
@@ -560,7 +602,7 @@ class MDNewton:
     keyword 'J'. Otherwise it will be calculated numerically.
 
     Please note that this method converges only locally. Especially for high-
-    dimensional systems it is not trvial to find a good starting point being
+    dimensional systems it is not trivial to find a good starting point being
     close enough to the root.
 
     It is recommended to use a faster, low-precision solver from SciPy [1] or
@@ -580,7 +622,7 @@ class MDNewton:
         assert x0.cols == 1, 'need a vector'
         self.x0 = x0
         if 'J' in kwargs:
-            self.J = kwarks['J']
+            self.J = kwargs['J']
         else:
             def J(*x):
                 return jacobian(f, x)
@@ -629,10 +671,10 @@ class MDNewton:
 # UTILITIES #
 #############
 
-str2solver = {'secant':Secant,'mnewton':MNewton, 'halley':Halley,
-              'muller':Muller, 'bisect':Bisection, 'illinois':Illinois,
-              'pegasus':Pegasus, 'anderson':Anderson, 'ridder':Ridder,
-              'anewton':ANewton, 'mdnewton':MDNewton}
+str2solver = {'newton':Newton, 'secant':Secant,'mnewton':MNewton,
+              'halley':Halley, 'muller':Muller, 'bisect':Bisection,
+              'illinois':Illinois, 'pegasus':Pegasus, 'anderson':Anderson,
+              'ridder':Ridder, 'anewton':ANewton, 'mdnewton':MDNewton}
 
 @extraprec(20)
 def findroot(f, x0, solver=Secant, tol=None, verbose=False, verify=True,
@@ -692,8 +734,8 @@ def findroot(f, x0, solver=Secant, tol=None, verbose=False, verify=True,
     compute `\pi` as the root of `\sin x` closest to `x_0 = 3`::
 
         >>> from mpmath import *
-        >>> mp.dps = 30
-        >>> print findroot(sin, 3)
+        >>> mp.dps = 30; mp.pretty = True
+        >>> findroot(sin, 3)
         3.14159265358979323846264338328
 
     The secant method can be used to find complex roots of analytic functions,
@@ -701,14 +743,14 @@ def findroot(f, x0, solver=Secant, tol=None, verbose=False, verify=True,
     (or else it will never leave the real line)::
 
         >>> mp.dps = 15
-        >>> print findroot(lambda x: x**3 + 2*x + 1, j)
+        >>> findroot(lambda x: x**3 + 2*x + 1, j)
         (0.226698825758202 + 1.46771150871022j)
 
     A nice application is to compute nontrivial roots of the Riemann zeta
     function with many digits (good initial values are needed for convergence)::
 
         >>> mp.dps = 30
-        >>> print findroot(zeta, 0.5+14j)
+        >>> findroot(zeta, 0.5+14j)
         (0.5 + 14.1347251417346937904572519836j)
 
     The secant method can also be used as an optimization algorithm, by passing
@@ -716,23 +758,25 @@ def findroot(f, x0, solver=Secant, tol=None, verbose=False, verify=True,
     minimum of the gamma function::
 
         >>> mp.dps = 20
-        >>> print findroot(lambda x: diff(gamma, x), 1)
+        >>> findroot(lambda x: diff(gamma, x), 1)
         1.4616321449683623413
 
     Finally, a useful application is to compute inverse functions, such as the
     Lambert W function which is the inverse of `w e^w`, given the first
     term of the solution's asymptotic expansion as the initial value. In basic
-    cases, this gives identical results to mpmath's builtin ``lambertw``
+    cases, this gives identical results to mpmath's built-in ``lambertw``
     function::
 
         >>> def lambert(x):
         ...     return findroot(lambda w: w*exp(w) - x, log(1+x))
         ...
         >>> mp.dps = 15
-        >>> print lambert(1), lambertw(1)
-        0.567143290409784 0.567143290409784
-        >>> print lambert(1000), lambert(1000)
-        5.2496028524016 5.2496028524016
+        >>> lambert(1); lambertw(1)
+        0.567143290409784
+        0.567143290409784
+        >>> lambert(1000); lambert(1000)
+        5.2496028524016
+        5.2496028524016
 
     Multidimensional functions are also supported::
 
@@ -749,6 +793,17 @@ def findroot(f, x0, solver=Secant, tol=None, verbose=False, verify=True,
 
     You can verify this by solving the system manually.
 
+    Please note that the following (more general) syntax also works::
+
+        >>> def f(x1, x2):
+        ...     return x1**2 + x2, 5*x1**2 - 3*x1 + 2*x2 - 3
+        ...
+        >>> findroot(f, (0, 0))
+        matrix(
+        [['-0.618033988749895'],
+         ['-0.381966011250105']])
+
+
     **Multiple roots**
 
     For multiple roots all methods of the Newtonian family (including secant)
@@ -756,7 +811,7 @@ def findroot(f, x0, solver=Secant, tol=None, verbose=False, verify=True,
 
         >>> f = lambda x: (x - 1)**99
         >>> findroot(f, 0.9, verify=False)
-        mpf('0.91807354244492868')
+        0.918073542444929
 
     Even for a very close starting point the secant method converges very
     slowly. Use ``verbose=True`` to illustrate this.
@@ -765,7 +820,7 @@ def findroot(f, x0, solver=Secant, tol=None, verbose=False, verify=True,
     the root's multiplicity::
 
         >>> findroot(f, -10, solver='mnewton')
-        mpf('1.0')
+        1.0
 
     This variant uses the first and second derivative of the function, which is
     not very efficient.
@@ -792,7 +847,7 @@ def findroot(f, x0, solver=Secant, tol=None, verbose=False, verify=True,
         error: 10.562244329955107759
         x: 1.0
         error: 7.8598304758094664213e-18
-        mpf('1.0')
+        1.0
 
 
     **Complex roots**
@@ -801,7 +856,8 @@ def findroot(f, x0, solver=Secant, tol=None, verbose=False, verify=True,
     even for real starting points very fast::
 
         >>> findroot(lambda x: x**4 + x + 1, (0, 1, 2), solver='muller')
-        mpc(real='0.72713608449119684', imag='0.93409928946052944')
+        (0.727136084491197 + 0.934099289460529j)
+
 
     **Intersection methods**
 
@@ -811,7 +867,7 @@ def findroot(f, x0, solver=Secant, tol=None, verbose=False, verify=True,
     with multiple roots and usually need a sign change to find a root::
 
         >>> findroot(lambda x: x**3, (-1, 1), solver='anderson')
-        mpf('0.0')
+        0.0
 
     Be careful with symmetric functions::
 
@@ -868,7 +924,7 @@ def findroot(f, x0, solver=Secant, tol=None, verbose=False, verify=True,
         # only one multidimensional solver available at the moment
         solver = MDNewton
         if not 'norm' in kwargs:
-            norm = lambda x: norm_p(x, mpf('inf'))
+            norm = lambda x: norm_(x, mpf('inf'))
             kwargs['norm'] = norm
         else:
             norm = kwargs['norm']
@@ -1006,3 +1062,9 @@ def steffensen(f):
         ffx = f(fx)
         return (x*ffx - fx**2) / (ffx - 2*fx + x)
     return F
+
+MultiPrecisionArithmetic.findroot = staticmethod(findroot)
+
+if __name__ == '__main__':
+    import doctest
+    doctest.testmod()
